defmodule DrainCloudCoreWeb.AuthController.RegisterRequest do
  @derive [Poison.Encoder, Jason.Encoder]

  defstruct login: "", password: ""

  alias __MODULE__, as: Request
  alias DrainCloudCore.Auth.Users, as: User
  alias DrainCloudCoreWeb.Errors.InvalidArgumentException, as: InvalidArgumentException

  def from_request(conn = %Plug.Conn{}) do
    # TODO remove Kernel.inspect calls
    :logger.debug("[from_request] incoming request: #{Kernel.inspect(conn.params)}")

    validate_and_build(conn.params)
  end

  def to_model(req = %Request{}) do
    %User{
      login: req.login,
      password: req.password
    }
  end

  defp validate_and_build(req) do
    validate(req)
    |> build
  end

  defp validate(req) do
    validate_login(req)
    |> validate_password
  end

  defp validate_login(req) do
    if String.length(Map.get(req, "login")) >= 3 do
      req
    else
      raise InvalidArgumentException, %{
        message: "login must be 3 symbols at minimum",
        args: ["login"]
      }
    end
  end

  defp validate_password(req) do
    if String.length(Map.get(req, "password")) >= 4 do
      req
    else
      raise InvalidArgumentException, %{
        message: "password must be 4 symbols at minimum",
        args: ["password"]
      }
    end
  end

  defp build(req) do
    %Request{
      login: Map.get(req, "login"),
      password: Map.get(req, "password") # TODO hash password
    }
  end

  defimpl String.Chars, for: __MODULE__ do
    def to_string(req) do
      case Jason.encode(req) do
        {:ok, str} -> str
        {:error, msg} -> raise "failed to encode register request to string: #{msg}"
      end
    end
  end
end
